/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  M051 serials input capture demo code

  Short P2.0 and P2.1 pins!
  P2.1(PWM1) outputs PWM signal and P2.0(Capture0) captures the PWM low and 
  high duty and show them on hyper-terminal. 
  The baud rate is 115200bps. The system clock is fixed as external crystal. 
  12MHz is required.

  Nuvoton Technoledge Corp. PU20 KEHsu
  22, Apr, 2010
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <q0> Capture0 Input Inverter Enable
// <h> Clock Divider (Capture scale should be smaller than PWM scale.) 
//    <o1.0..2> Capture0 Clock Divider
//                  <4=> 1/1 <0=> 1/2 <1=> 1/4 <2=> 1/8 <3=> 1/16
//    <o1.4..6> PWM1 Clock Divider
//                  <4=> 1/1 <0=> 1/2 <1=> 1/4 <2=> 1/8 <3=> 1/16
// </h>
//
//-------- <<< end of configuration section >>> ------------------------------
*/

#include <stdio.h>
#include <stdint.h>
#include "common.h"
#include "Register_Bit.h"
#include "M051.h"
#include "UART.h"

#define CCPTURE_INPUT_INVERSE   0
#define CLOCK_DIVIDER           0x00000033 

/*-----------------------------------------------------------------------------
  Function:		PWMA_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	PDMA ISR routine, Calculate duty ratio of Capture0.                                    
 *-----------------------------------------------------------------------------*/
void PWMA_IRQHandler(void)
{
    uint16_t uiLow_Pulse, uiHigh_Pulse;
	uint32_t u32status;

	u32status = CCR0A;
    CCR0A |= CAPIF0;                      //Capture0 flag clearing by writing "1"

    uiLow_Pulse = CNR0A - CRLR0A;         //Calculate low pulse width                             
    uiHigh_Pulse = CNR0A - CFLR0A;        //Calculate high pulse width
     
    if (u32status & CFLRI0)               //If a PWM falling edge occure, it means a complete PWM period outputs, show the duty cycle on hyper terminal 
        printf("Low Duty: %4d/%d, High Duty: %4d/%d\n", uiLow_Pulse, uiLow_Pulse + uiHigh_Pulse, uiHigh_Pulse, uiLow_Pulse + uiHigh_Pulse);
}

main(void)
{	
    Un_Lock_Reg();                                          
    
	PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

    Lock_Reg();

	UART_Init();                          //Initial UART, baud rate = 115200bps @ crystal 11.0592MHz
	
	printf("press Enter to start...\n");
	getchar();

    P2_MFP &= ~(P20_AD8_PWM0 | P21_AD9_PWM1);   
	P2_MFP |= (PWM0 | PWM1);              //Switch P2.0,1 to PWM0,1 output
    P2_PMD &= ~(Px0_PMD | Px1_PMD);    
	P2_PMD |= Px1_OUT;		
	P2_PMD |= Px0_QB;				  	  //Configure Capture0 as "quasi bi-direction mode", PWM1 output pins "push-pull" mode

    APBCLK |= PWM01_CLKEN;                //Enable PWM0,1 clock
	CLKSEL1 = (CLKSEL1 & (~PWM01_CLK)) | PWM01_12M;	 //Select external crystal as Capture0 and PWM1 clock source
                  
    PPRA = 0x000000FF;                    //Select PWM0,1 clock pre-scalar 256
    CSRA = CLOCK_DIVIDER;                 //Select PWM0~3 clock divider
    PCRA |= CH1_AU_RL;
	PCRA |= CH0_AU_RL;                    //Set Capture0 and PWM1 under auto-reload (continuous) mode 
    //********** Note that Capture0 also needs auto-reload! ***************
	
	CNR0A = 0xffff;                       //Any but suitable value for Capture0 down counter
    
	CCR0A &= ~INV0_ON; 	  				  
    CCR0A |= CCPTURE_INPUT_INVERSE;		  //CCPTURE_INPUT_INVERSE defines input phase
	
	CCR0A |= CAPCH0EN;					  //Enable Capture function pin
	CAPENRA |= CAPCH0_ON;                 //Enable Capture0 channel input valid
                                    
    PIERA |= PWMIE0;                      //Enable PWM interrupt for down-counter equal zero.
    CCR0A |= CFL_IE0;					  //Enable Capture falling edge interrupt
	CCR0A |= CRL_IE0;					  //Enable Capture rising edge interrupt
	NVIC_ISER |= PWMA_INT;                //Enable NVIC PWM0~3 interrupt

	CNR1A = 255;                          //Fixed PWM1 period 256 clocks
	CMR1A = 63;

    POEA = PWM1_OE;                       //PWM1 output enable
    PCRA |= CH1EN;						  //Enable PWM1 down counter
	PCRA |= CH0EN;		                  //Enable PWM0 down counter

    while(1);		
}
