/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  Nuvoton Technoledge Corp creadted by PU20 
  This demo code is used to show the GPIO function 
  P0/1/2/3/4 will output Hi-Lo.....       
                                            
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
// <e5> GPIO Configuration
//    <o6.0..31> P0 Configuration (P0_PMD)
//                  <0x0000=> P0 INPUT Mode
//                  <0x5555=> P0 OUTPUT Mode
//                  <0xAAAA=> P0 Open-Drain Mode
//                  <0xFFFF=> P0 Quasi-bidirection Mode
//
//    <o7.0..31> P1 Configuration (P1_PMD)
//                  <0x0000=> P1 INPUT Mode
//                  <0x5555=> P1 OUTPUT Mode
//                  <0xAAAA=> P1 Open-Drain Mode
//                  <0xFFFF=> P1 Quasi-bidirection Mode
//
//    <o8.0..31> P2 Configuration (P2_PMD)
//                  <0x0000=> P2 INPUT Mode
//                  <0x5555=> P2 OUTPUT Mode
//                  <0xAAAA=> P2 Open-Drain Mode
//                  <0xFFFF=> P2 Quasi-bidirection Mode
//
//    <o9.0..31> P3 Configuration (P3_PMD)
//                  <0x0000=> P3 INPUT Mode
//                  <0x5555=> P3 OUTPUT Mode
//                  <0xAAAA=> P3 Open-Drain Mode
//                  <0xFFFF=> P3 Quasi-bidirection Mode
// </e5>
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           1
#define CLOCK_EN              0xF
#define PLL_Engine_Enable     1 
#define PLL_SEL               0x00000000 
#define CLOCK_SEL             0x0
#define GPIO_SETUP            1
#define P0_MODE               0x5555
#define P1_MODE     	      0x5555
#define P2_MODE 	          0x5555
#define P3_MODE	              0x5555

#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"

void TMR0_Delay1ms(uint32_t ulCNT);
void Timer0_Init(void);

/*-----------------------------------------------------------------------------
  Function:		TMR0_Delay1ms                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulCNT:	delay time, unit is millisecond.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer0 used for a delay function.                                    
 *-----------------------------------------------------------------------------*/
void TMR0_Delay1ms(uint32_t ulCNT)
{
    TCMPR0 = 12000;         //Fosc=12MHz, so 12000000/12000=1000Hz=1ms
    setTMR0_CEN;            //Start timer0
    while (ulCNT != 1)
    {
        while ((TISR0&TMR_TIF) != TMR_TIF); //check TIF0
        TISR0 |= TMR_TIF;   //Clear TIF0
        ulCNT --;
    }
    clrTMR0_CEN;            //Stop timer0
}

/*-----------------------------------------------------------------------------
  Function:		Timer0_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer0, period mode.                                    
 *-----------------------------------------------------------------------------*/
void Timer0_Init()
{
    TMR0_Clock_EN;
	TMR0ClkSource_ex12MHz;
    TCSR0  = 0x00000001;    //Pre-Scaler
    
    setTMR0_PERIOD;
    setTMR0_IE;             //Timer0 interrupt enable
    setTMR0_CRST;           //Reset the timer/counter0, after set, this bit will be clear by H/W
}

main(void)
{
    Un_Lock_Reg();
	PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

    #if GPIO_SETUP == 1
    {
        P0_PMD = P0_MODE;
        P1_PMD = P1_MODE;
        P2_PMD = P2_MODE;
        P3_PMD = P3_MODE;
    }
    #endif
    
    Timer0_Init();

    while(1)
    {
        P0_DOUT = 0x00;
        P1_DOUT = 0x00;
        P2_DOUT = 0x00;
        P3_DOUT = 0x00;
        TMR0_Delay1ms(50);
        
        P0_DOUT = 0xFF;
        P1_DOUT = 0xFF;
        P2_DOUT = 0xFF;
        P3_DOUT = 0xFF;
        TMR0_Delay1ms(50);
    }
}
