/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------------
  Nuvoton Technoledge Corp creadted by PU20 
 
  XTAL = 12MHz
  if AP_Program_LD = 1, this F/W (@APROM) will demo APROM access LDROM, CONFIG0.7=1
  if AP_Program_LD = 0, this F/W (@LDROM) will demo LDROM access APROM, CONFIG0.7=0
 *--------------------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
// <q5> AP Program LD 
//-------- <<< end of configuration section >>> ------------------------------
*/

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "Uart.h"
#include "Macro_SystemClock.h"
#include "ISP.h"

#define CLOCK_SETUP             1
#define CLOCK_EN                0xF
#define PLL_Engine_Enable       0 
#define PLL_SEL                 0x00000001 
#define CLOCK_SEL               0x0
#define AP_Program_LD           0

#define PAGE_Size               512
#define APROM_Start_Address     0x00000000 //A21=0,A20=0
#define LDROM_Start_Address     0x00100000 //A21=0,A20=1
#define CONFIG_Start_Address    0x00300000 //A21=1,A20=1
#define APROM_Size              0x00010000 //128K Bytes
#define LDROM_Size              0x00001000 //4096 Bytes

/*-----------------------------------------------------------------------------
  Function:		ISP_Triger                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Start ISP operation.                                   
 *-----------------------------------------------------------------------------*/
void ISP_Triger(void)
{
    ISPTRG |= ISPGO;
    while((ISPTRG&ISPGO) == ISPGO);
}

/*-----------------------------------------------------------------------------
  Function:		ISP_Enable                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Enable ISP operation.                                   
 *-----------------------------------------------------------------------------*/
void ISP_Enable(void)
{
    Un_Lock_Reg();
    ISPCON |= ISPEN;
}

/*-----------------------------------------------------------------------------
  Function:		ISP_Disable                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Disable ISP operation.                                   
 *-----------------------------------------------------------------------------*/
void ISP_Disable(void)
{
    Un_Lock_Reg();
    ISPCON &= ~ISPEN;
}

/*-----------------------------------------------------------------------------
  Function:		ISP_Read_CID                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	Device CID.                                                                                      
  Description:                                                                                            
               	ISP read CID.                                   
 *-----------------------------------------------------------------------------*/
uint32_t ISP_Read_CID(void)
{
    uint32_t ulTmp;
    
    ISP_Enable();
    ISPCMD = READ_CID;
    ISP_Triger();
    ulTmp = ISPDAT;
    ISP_Disable();
    return ulTmp;
}

/*-----------------------------------------------------------------------------
  Function:		ISP_Read_DID                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	Device CID.                                                                                      
  Description:                                                                                            
               	ISP read DID.                                   
 *-----------------------------------------------------------------------------*/
uint32_t ISP_Read_DID(void)
{
    uint32_t ulTmp;
    
    ISP_Enable();
    ISPCMD = READ_DID;
    ISP_Triger();
    ulTmp = ISPDAT;
    ISP_Disable();
    return ulTmp;
}

/*-----------------------------------------------------------------------------
  Function:		Show_LD_Erase_Verify_Error                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Print error message when an error occurs when erasing LDROM.                                   
 *-----------------------------------------------------------------------------*/
void Show_LD_Erase_Verify_Error(void)
{
	printf("Error when Erasing LDROM\n");
    while(1);
}

/*-----------------------------------------------------------------------------
  Function:		Show_LD_Program_Verify_Error                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Print error message when an error occurs when Programing LDROM.                                   
 *-----------------------------------------------------------------------------*/
void Show_LD_Program_Verify_Error(void)
{
	printf("Error when Programing LDROM\n");
    while(1);
}

/*-----------------------------------------------------------------------------
  Function:		LDROM_RW_Enable                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	ISP update LDROM is enabled.                                   
 *-----------------------------------------------------------------------------*/
void LDROM_RW_Enable(void)
{
    Un_Lock_Reg();
    ISPCON |= LDUEN;
}

/*-----------------------------------------------------------------------------
  Function:		LDROM_Erase                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Erase LDROM.                                   
 *-----------------------------------------------------------------------------*/
void LDROM_Erase(void)
{
    uint32_t ulCount;
    uint8_t ucPageCount;    

    ISP_Enable();
    LDROM_RW_Enable();
    ISPCMD = PAGE_ERASE;

    ucPageCount = (uint8_t)(LDROM_Size/PAGE_Size); //4096/512=8 pages
    for(ulCount=0;ulCount<(uint32_t)ucPageCount;ulCount++) 
    {
        ISPADR = (ulCount*PAGE_Size+LDROM_Start_Address);
        ISP_Triger();
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		LDROM_Program                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulDATA: Data to be written to LDROM                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	LDROM will be written with 0x55555555,0xAAAAAAAA,0x55555555,
				0xAAAAAAAA......                                   
 *-----------------------------------------------------------------------------*/
void LDROM_Program(uint32_t ulDATA)
{
    uint32_t ulCount;    

    ISP_Enable();
    LDROM_RW_Enable();
    ISPCMD = PROGRAM;

    //0x0400=1024, 4Kx8=1Kx32=1024x32
    for(ulCount=0;ulCount<(LDROM_Size/4);ulCount++) 
    {
        ISPADR = (ulCount*4+LDROM_Start_Address);
        ISPDAT = ulDATA;
        ISP_Triger();
        ulDATA = ~ulDATA;
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		LDROM_Erase_Verify                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Verify that Erase LDROM success or not.                                   
 *-----------------------------------------------------------------------------*/
void LDROM_Erase_Verify(void)
{
    uint32_t ulCount;

    ISP_Enable();
    LDROM_RW_Enable();
    ISPCMD = READ;

    //0x0400=1024, 4Kx8=1Kx32=1024x32
    for(ulCount=0;ulCount<(LDROM_Size/4);ulCount++) 
    {
        ISPADR = (ulCount*4+LDROM_Start_Address);
        ISP_Triger();
        if(ISPDAT != 0xFFFFFFFF)
        {
            Show_LD_Erase_Verify_Error();
            return;
        }
        ISPDAT = 0;
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		LDROM_Program_Verify                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulDATA: Used to compare with Porgrammed data in LDROM.                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Verify that Program LDROM success or not.                                   
 *-----------------------------------------------------------------------------*/
void LDROM_Program_Verify(uint32_t ulDATA)
{
    uint32_t ulCount;

    ISP_Enable();
    LDROM_RW_Enable();
    ISPCMD = READ;

    //0x0400=1024, 4Kx8=1Kx32=1024x32
    for(ulCount=0;ulCount<(LDROM_Size/4);ulCount++) 
    {
        ISPADR = (ulCount*4+LDROM_Start_Address);
        ISP_Triger();
        if(ISPDAT != ulDATA)
        {
            Show_LD_Program_Verify_Error();
            return;
        }
        ulDATA = ~ulDATA;
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		Show_AP_Erase_Verify_Error                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Print error message when an error occurs when erasing APROM.                                   
 *-----------------------------------------------------------------------------*/
void Show_AP_Erase_Verify_Error(void)
{
	printf("Error when Erasing APROM\n");
    while(1);
}

/*-----------------------------------------------------------------------------
  Function:		Show_AP_Program_Verify_Error                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Print error message when an error occurs when Programing APROM.                                   
 *-----------------------------------------------------------------------------*/
void Show_AP_Program_Verify_Error(void)
{
	printf("Error when Programing APROM\n");
    while(1);
}

/*-----------------------------------------------------------------------------
  Function:		APROM_Erase                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Erase APROM.                                   
 *-----------------------------------------------------------------------------*/
void APROM_Erase(void)
{
    uint32_t ulCount;
    uint16_t uiPageCount;    

    ISP_Enable();
    ISPCMD = PAGE_ERASE;

    uiPageCount = (uint16_t)(APROM_Size/PAGE_Size); //4096/512=8 pages
    for(ulCount=0;ulCount<(uint32_t)uiPageCount;ulCount++) 
    {
        ISPADR = (ulCount*PAGE_Size+APROM_Start_Address);
        ISP_Triger();
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		APROM_Program                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulDATA: Data to be written to APROM                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	APROM will be written with 0x55555555,0xAAAAAAAA,0x55555555,
				0xAAAAAAAA......                                   
 *-----------------------------------------------------------------------------*/
void APROM_Program(uint32_t ulDATA)
{
    uint32_t ulCount;    

    ISP_Enable();
    ISPCMD = PROGRAM;

    for(ulCount=0;ulCount<(APROM_Size/4);ulCount++) 
    {
        ISPADR = (ulCount*4+APROM_Start_Address);
        ISPDAT = ulDATA;
        ISP_Triger();
        ulDATA = ~ulDATA;
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		APROM_Erase_Verify                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Verify that Erase APROM success or not.                                   
 *-----------------------------------------------------------------------------*/
void APROM_Erase_Verify(void)
{
    uint32_t ulCount;

    ISP_Enable();
    ISPCMD = READ;

    for(ulCount=0;ulCount<(APROM_Size/4);ulCount++) 
    {
        ISPADR = (ulCount*4+APROM_Start_Address);
        ISP_Triger();
        if(ISPDAT != 0xFFFFFFFF)
        {
            Show_AP_Erase_Verify_Error();
            return;
        }
        ISPDAT = 0;
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		APROM_Program_Verify                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulDATA: Data to be compared with programmed APROM data.                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Verify that Program APROM success or not.                                   
 *-----------------------------------------------------------------------------*/
void APROM_Program_Verify(uint32_t ulDATA)
{
    uint32_t ulCount;

    ISP_Enable();
    ISPCMD = READ;

    for(ulCount=0;ulCount<(APROM_Size/4);ulCount++) 
    {
        ISPADR = (ulCount*4+APROM_Start_Address);
        ISP_Triger();
        if(ISPDAT != ulDATA)
        {
            Show_AP_Program_Verify_Error();
            return;
        }
        ulDATA = ~ulDATA;
    }
    
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		CONFIG_RW_Enable                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	ISP update CONFIG area is enabled.                                   
 *-----------------------------------------------------------------------------*/
void CONFIG_RW_Enable(void)
{
    Un_Lock_Reg();
    ISPCON |= CFGUEN;
}

/*-----------------------------------------------------------------------------
  Function:		CONFIG_Erase                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Erase CONFIG area.                                   
 *-----------------------------------------------------------------------------*/
void CONFIG_Erase(void)
{
    ISP_Enable();
    CONFIG_RW_Enable();
    ISPCMD = PAGE_ERASE;
    ISPADR = CONFIG_Start_Address;
    ISP_Triger();
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		CONFIG0_Program                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulDATA: Data to be written to CONFIG0                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Program CONFIG0 area.                                   
 *-----------------------------------------------------------------------------*/
void CONFIG0_Program(uint32_t ulDATA)
{
    ISP_Enable();
    CONFIG_RW_Enable();
    ISPCMD = PROGRAM;
    ISPADR = CONFIG_Start_Address+0x00;
    ISPDAT = ulDATA;
    ISP_Triger();
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		CONFIG1_Program                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulDATA: Data to be written to CONFIG1                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Program CONFIG1 area.                                   
 *-----------------------------------------------------------------------------*/
void CONFIG1_Program(uint32_t ulDATA)
{
    ISP_Enable();
    CONFIG_RW_Enable();
    ISPCMD = PROGRAM;
    ISPADR = CONFIG_Start_Address+0x04;
    ISPDAT = ulDATA;
    ISP_Triger();
    ISP_Disable();        
}

/*-----------------------------------------------------------------------------
  Function:		CONFIG0_Read                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                   
  Returns:                                                                                                
               	Value of CONFIG0                                                                                      
  Description:                                                                                            
               	Read CONFIG0 area.                                   
 *-----------------------------------------------------------------------------*/
uint32_t CONFIG0_Read(void)
{
    uint32_t ulTmp;
    
    ISP_Enable();
    ISPCMD = READ;
    ISPADR = CONFIG_Start_Address+0x00;
    ISP_Triger();
    ulTmp = ISPDAT;
    ISP_Disable();
    return ulTmp;
}

/*-----------------------------------------------------------------------------
  Function:		CONFIG1_Read                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                   
  Returns:                                                                                                
               	Value of CONFIG1                                                                                      
  Description:                                                                                            
               	Read CONFIG1 area.                                   
 *-----------------------------------------------------------------------------*/
uint32_t CONFIG1_Read(void)
{
    uint32_t ulTmp;
    
    ISP_Enable();
    ISPCMD = READ;
    ISPADR = CONFIG_Start_Address+0x04;
    ISP_Triger();
    ulTmp = ISPDAT;
    ISP_Disable();
    return ulTmp;
}

main(void)
{
    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

	UART_Init();

	printf("Press Enter to start...\n");
	getchar();

	printf("Device CID is 0x%X\n", ISP_Read_CID());
	printf("Device DID is 0x%X\n", ISP_Read_DID());

    #if AP_Program_LD == 1
		printf("APROM access to LDROM\n");
        CONFIG_Erase();
        CONFIG0_Program(0xF8FFFFFF);
        CONFIG1_Program(0xEFFFFFFF);
        LDROM_Erase();
        LDROM_Erase_Verify();
        LDROM_Program(0x55555555);
        LDROM_Program_Verify(0x55555555);
    #else
		printf("LDROM access to APROM\n");
        CONFIG_Erase();
        /**************************************************************/
        CONFIG0_Program(0xF8FFFF7F);        //Note: CONFIG0 = 0xF8FFFF7F
        /**************************************************************/
        CONFIG1_Program(0xEFFFFFFF);
        APROM_Erase();
        APROM_Erase_Verify();
        APROM_Program(0x55555555);
        APROM_Program_Verify(0x55555555);
    #endif

	printf("ISP test is successful\n");

    while(1);
}
