/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  M051 serials SPI demo code

  Winbond SPI flash W25X10L, W25X20L, W25X40L and W25X80L are used as the slave 
  device.
  Please Set Hyper Terminal: 115200, 8, None, 1, None
  Connect P1.4 to SPI Flash Pin.CS     
  Connect P1.7 to SPI Flash Pin.CLK
  Connect P1.6 to SPI Flash Pin.DIO
  Connect P1.5 to SPI Flash Pin.DO

  Put key0,1,2,3 to run function.

  Nuvoton Technoledge Corp. PU20 HYLi
  16, Apr, 2010
  XTAL = 12MHz, BaudRate = 115200bps
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> External Crystal 12MHz
//                  <q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//  </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
//  <e5> SPI Clock Divider Enable
//      <o6.0..15> SPI0 Clock Divider<0-65535>
//  </e5>
//
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           1
#define CLOCK_EN              0x0000001F
#define PLL_Engine_Enable     0 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x0
#define SPI_DIVIDER_EN        1
#define SPI0_DIVIDER_VALUE    0x00000003

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"
#include "Retarget.h"
#include "Uart.h"
#include "Common.h"
#include "L_Function.h"
#include "L_SPI.h"

void Define_Value(void);

uint32_t ucgSPI0_Divider_Value;

main(void)
{
    uint8_t getkey;

    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

    Define_Value();
    Timer0_Init();          			  		//Timer0 Use External 12MHz Crystal  
    UART_Init();
    
    printf("\nSPI Test Begin...............\n");
    SPI0_Master_Init();

    while(1)
    {   
        printf("Key 0: Get ID\n");
        printf("Key 1: Page0 Read Data\n");
        printf("Key 2: Program Page0 Data\n");
        printf("Key 3: Erase All Chip\n");
        printf("Put key0~3 run fucntion\n");
        getkey = getchar();
        printf("\n");
        switch(getkey)
        {
            case '0':        
               Get_SPI_Flash_ID();
               break;
            case '1':
                Read_Page0_Data();
                break;
            case '2':
                Program_Page0_Data();
                break;
            case '3':
                Erase_All_Chip();
                break;
        }//end switch(key)
        printf("\n");
    }//end while(1)

}

/*-----------------------------------------------------------------------------
  Function:		Define_Value                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Set SPI0 divider value.                                    
 *-----------------------------------------------------------------------------*/
void Define_Value(void)
{
    ucgSPI0_Divider_Value = 0x00000000;
    #if SPI_DIVIDER_EN
        ucgSPI0_Divider_Value = SPI0_DIVIDER_VALUE;
    #endif
}

