/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
   
/*-----------------------------------------------------------------------------
  M051 serials SPI demo code

  SPI Loop Back Test.
  Use SPI0 Master Mode to SPI1 Slave Mode.
  Please Set Hyper Terminal: 115200, 8, None, 1, None
  Connect P0.4 to P1.4 
  Connect P0.5 to P1.5 
  Connect P0.6 to P1.6 
  Connect P0.7 to P1.7 

  This demo use SPI0(master) to send a data 0x55 to SPI1(slave),
  and then recive a data 0xDA from SPI1.

  Nuvoton Technoledge Corp. PU20 HYLi
  21, Apr, 2010
  XTAL = 12MHz
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> External Crystal 12MHz
//                  <q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//  </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
//  <e5> SPI Clock Divider Enable
//      <o6.0..15> SPI0 Clock Divider<0-65535> 
//      <o7.0..15> SPI1 Clock Divider<0-65535>
//  </e5>
//
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           1
#define CLOCK_EN              0x0000001F
#define PLL_Engine_Enable     0 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x0
#define SPI_DIVIDER_EN        1
#define SPI0_DIVIDER_VALUE    0x00000002
#define SPI1_DIVIDER_VALUE    0x00000002

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"
#include "Retarget.h"
#include "Uart.h"
#include "Common.h"
#include "Macro_SPI.h"
#include "L_SPI.h"
#include "L_Function.h"

void Define_Value(void);

uint32_t ucgSPI0_Divider_Value,ucgSPI1_Divider_Value;

main(void)
{   
	uint8_t ucTemp=0;
    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock
    
    Define_Value();
    Timer0_Init();          	  		  		//Timer0 Use External 12MHz Crystal Oscillator  
    UART_Init();
    
    SPI0_Master_Init();
    SPI1_Slave_Init();
    printf("ucTemp = %X...............\n",ucTemp);
    printf("\nSPI Loop Test Begin...............\n");
    
    while(1)
    {
        printf("Master will send 0x55 data to slave and receive 0xDA data\n");
        printf("Put AnyKey to Start Test\n");
        getchar();
        Get_Slave_ID();
        Print_Get_Data();
        printf("\n\n");
    }
}

/*-----------------------------------------------------------------------------
  Function:		Define_Value                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Set SPI0 divider value.                                    
 *-----------------------------------------------------------------------------*/
void Define_Value(void)
{
    ucgSPI0_Divider_Value = 0x00000000;
    ucgSPI1_Divider_Value = 0x00000000;
    #if SPI_DIVIDER_EN
        ucgSPI0_Divider_Value = SPI0_DIVIDER_VALUE;
        ucgSPI1_Divider_Value = SPI1_DIVIDER_VALUE;
    #endif
}
