/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  Nuvoton Technoledge Corp creadted by PU20  
  XTAL = 12MHz, BPS = 115200
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
//-------- <<< end of configuration section >>> ------------------------------
*/

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "UART.h"
#include "Retarget.h"
#include "Macro_SystemClock.h"

#define CLOCK_SETUP           0
#define CLOCK_EN              0xF
#define PLL_Engine_Enable     1 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x2

#define RXBUFSIZE 64


/* Global variables */                                                                                       
volatile uint8_t comRbuf[RXBUFSIZE];
volatile uint16_t comRbytes = 0;		/* Available receiving bytes */
volatile uint16_t comRhead 	= 0;
volatile uint16_t comRtail 	= 0;
volatile int32_t g_bWait 	= 1;

/*-----------------------------------------------------------------------------
  Function:		UART0_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	UART0 ISR routine.                                    
 *-----------------------------------------------------------------------------*/
 void UART0_IRQHandler(void)
{
 	uint8_t bInChar[1]={0xFF};
	
	if(UA0_ISR & RDA_INT)
	{
		printf("\nInput:");
		
		/* Get all the input characters */
		while(UA0_ISR & RDA_IF) 
		{
		    /* Check RX empty */
			while (UA0_FSR & RX_EMPTY);
			bInChar[0] = UA0_RBR;					  

			printf("%c ", bInChar[0]);
			
			if(bInChar[0] == '0')	
			{	
				g_bWait = 0;
			}
		
			/* Check if buffer full */
			if(comRbytes < RXBUFSIZE)
			{
				/* Enqueue the character */
				comRbuf[comRtail] = bInChar[0];
				comRtail = (comRtail == (RXBUFSIZE-1)) ? 0 : (comRtail+1);
				comRbytes++;
			}			
		}
		printf("\nTransmission Test:");
	}
	else if(UA0_ISR & THRE_INT)
	{   
		   
        uint16_t tmp;
        tmp = comRtail;
		if(comRhead != tmp)
		{
			bInChar[0] = comRbuf[comRhead];
			/* Wait Tx empty */
			while ((UA0_FSR&TX_EMPTY) == 0x00);
			UA0_THR = bInChar[0];						
			comRhead = (comRhead == (RXBUFSIZE-1)) ? 0 : (comRhead+1);
			comRbytes--;
		}
	}
}

int32_t main()
{
	/* UART Clock is ext12MHz */
	UARTClkSource_ex12MHZ;  			  
    Un_Lock_Reg();
    
	/* Enable external 12M Crystal */
	PWRCON |= XTL12M_EN;
	/* Wait until 12M clock is stable. */
	while((CLKSTATUS & XTL12M_STB) == 0); 	
	
	/* Set external crystal as the system clock	*/
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;

    UART_Init();

	printf("\nUART Sample Demo. (Press '0' to exit)\n");

	/* Enable Interrupt */
	UA0_IER	|= (RDA_IEN	| THRE_IEN | RLS_IEN);
	NVIC_ISER |= UART0_INT;

	while(g_bWait);	
		
	/* Disable Interrupt */
	UA0_IER	&= ~(RDA_IEN	| THRE_IEN | RLS_IEN);

	/* Disable I & F bit */ 
	NVIC_ISER &= ~UART0_INT;		
	
	printf("\nUART Sample Demo End.\n");
	
	return 1;
}	
